@@ -33,6 +33,7 @@
  *
  * After dequeue, processing is as follows:
  * - processBatchMain
+ * - processMsgMain (direct entry for DIRECT queue!)
  * - ...
  *
  * MORE ON PROCESSING, QUEUES and FILTERING
@@ -63,7 +64,7 @@
  * beast.
  * rgerhards, 2011-06-15
  *
- * Copyright 2007-2018 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2019 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -95,9 +96,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#ifdef _AIX
-#include <pthread.h>
-#endif
 #include <json.h>
 
 #include "rsyslog.h"
@@ -122,9 +120,8 @@
 #ifdef _AIX
 #define cs legacy_cs
 #endif
-#if !defined(_AIX)
-#pragma GCC diagnostic ignored "-Wswitch-enum"
-#endif
+
+PRAGMA_INGORE_Wswitch_enum
 
 #ifndef O_LARGEFILE
 #define O_LARGEFILE 0
@@ -211,6 +208,7 @@ static struct cnfparamdescr cnfparamdescr[] = {
 	{ "action.reportsuspensioncontinuation", eCmdHdlrBinary, 0 },
 	{ "action.resumeintervalmax", eCmdHdlrPositiveInt, 0 },
 	{ "action.resumeinterval", eCmdHdlrInt, 0 },
+	{ "action.externalstate.file", eCmdHdlrString, 0 },
 	{ "action.copymsg", eCmdHdlrBinary, 0 }
 };
 static struct cnfparamblk pblk =
@@ -357,6 +355,7 @@ rsRetVal actionDestruct(action_t * const pThis)
 	pthread_mutex_destroy(&pThis->mutAction);
 	pthread_mutex_destroy(&pThis->mutWrkrDataTable);
 	free((void*)pThis->pszErrFile);
+	free((void*)pThis->pszExternalStateFile);
 	free(pThis->pszName);
 	free(pThis->ppTpl);
 	free(pThis->peParamPassing);
@@ -399,6 +398,7 @@ rsRetVal actionConstruct(action_t **ppThis)
 	pThis->iResumeRetryCount = 0;
 	pThis->pszName = NULL;
 	pThis->pszErrFile = NULL;
+	pThis->pszExternalStateFile = NULL;
 	pThis->fdErrFile = -1;
 	pThis->bWriteAllMarkMsgs = 1;
 	pThis->iExecEveryNthOccur = 0;
@@ -684,6 +684,52 @@ static void actionCommitted(action_t * const pThis, wti_t * const pWti)
 }
 
 
+/* set action state according to external state file (if configured)
+*/
+static rsRetVal
+checkExternalStateFile(action_t *__restrict__ const pThis)
+{
+	char filebuf[1024];
+	int fd = -1;
+	int r;
+	DEFiRet;
+
+	DBGPRINTF("checking external state file\n");
+
+	if(pThis->pszExternalStateFile == NULL) {
+		FINALIZE;
+	}
+
+	fd = open(pThis->pszExternalStateFile, O_RDONLY|O_CLOEXEC);
+	if(fd == -1) {
+		dbgprintf("could not read external state file\n");
+		FINALIZE;
+	}
+
+	r = read(fd, filebuf, sizeof(filebuf) - 1);
+	if(r < 1) {
+		dbgprintf("checkExternalStateFile read() returned %d\n", r);
+		FINALIZE;
+	}
+
+	filebuf[r] = '\0';
+	dbgprintf("external state file content: '%s'\n", filebuf);
+	if(!strcmp(filebuf, "SUSPENDED")) {
+		LogMsg(0, RS_RET_SUSPENDED, LOG_WARNING,
+		      "action '%s' suspended (module '%s') by external state file",
+		      pThis->pszName, pThis->pMod->pszName);
+		ABORT_FINALIZE(RS_RET_SUSPENDED);
+	}
+
+finalize_it:
+	if(fd != -1) {
+		close(fd);
+	}
+	DBGPRINTF("done checking external state file, iRet=%d\n", iRet);
+	RETiRet;
+}
+
+
 /* we need to defer setting the action's own bReportSuspension state until
  * after the full config has been processed. So the most simple case to do
  * that is here. It's not a performance problem, as it happens infrequently.
@@ -783,7 +829,7 @@ actionSuspend(action_t * const pThis, wti_t * const pWti)
  * kind of facility: in the first place, the module should return a proper indication
  * of its inability to recover. -- rgerhards, 2010-04-26.
  */
-static rsRetVal
+static rsRetVal ATTR_NONNULL()
 actionDoRetry(action_t * const pThis, wti_t * const pWti)
 {
 	int iRetries;
@@ -797,7 +843,10 @@ actionDoRetry(action_t * const pThis, wti_t * const pWti)
 	while((*pWti->pbShutdownImmediate == 0) && getActionState(pWti, pThis) == ACT_STATE_RTRY) {
 		DBGPRINTF("actionDoRetry: %s enter loop, iRetries=%d, ResumeInRow %d\n",
 			pThis->pszName, iRetries, getActionResumeInRow(pWti, pThis));
-		iRet = pThis->pMod->tryResume(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
+		iRet = checkExternalStateFile(pThis);
+		if(iRet == RS_RET_OK) {
+			iRet = pThis->pMod->tryResume(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
+		}
 		DBGPRINTF("actionDoRetry: %s action->tryResume returned %d\n", pThis->pszName, iRet);
 		if((getActionResumeInRow(pWti, pThis) > 9) && (getActionResumeInRow(pWti, pThis) % 10 == 0)) {
 			bTreatOKasSusp = 1;
@@ -1023,10 +1072,9 @@ finalize_it:
 
 
 /* the #pragmas can go away when we have disable array-passing mode */
-#if !defined(_AIX)
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-align"
-#endif
+
+PRAGMA_DIAGNOSTIC_PUSH
+PRAGMA_IGNORE_Wcast_align
 void
 releaseDoActionParams(action_t *__restrict__ const pAction, wti_t *__restrict__ const pWti, int action_destruct)
 {
@@ -1061,9 +1109,8 @@ releaseDoActionParams(action_t *__restrict__ const pAction, wti_t *__restrict__
 
 	return;
 }
-#if !defined(_AIX)
-#pragma GCC diagnostic pop
-#endif
+
+PRAGMA_DIAGNOSTIC_POP
 
 
 /* This is used in resume processing. We only finally know that a resume
@@ -1159,12 +1206,15 @@ actionCallCommitTransaction(action_t * const pThis,
 	DBGPRINTF("entering actionCallCommitTransaction[%s], state: %s, nMsgs %u\n",
 		  pThis->pszName, getActStateName(pThis, pWti), nparams);
 
-	iRet = pThis->pMod->mod.om.commitTransaction(
-		    pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData,
-		    iparams, nparams);
-	DBGPRINTF("actionCallCommitTransaction[%s] state: %s "
-		"mod commitTransaction returned %d\n",
-		pThis->pszName, getActStateName(pThis, pWti), iRet);
+	iRet = checkExternalStateFile(pThis);
+	if(iRet == RS_RET_OK) {
+		iRet = pThis->pMod->mod.om.commitTransaction(
+			    pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData,
+			    iparams, nparams);
+		DBGPRINTF("actionCallCommitTransaction[%s] state: %s "
+			"mod commitTransaction returned %d\n",
+			pThis->pszName, getActStateName(pThis, pWti), iRet);
+	}
 	iRet = handleActionExecResult(pThis, pWti, iRet);
 	RETiRet;
 }
@@ -1508,7 +1558,7 @@ actionCommitAllDirect(wti_t *__restrict__ const pWti)
 }
 
 /* process a single message. This is both called if we run from the
- * cosumer side of an action queue as well as directly from the main
+ * consumer side of an action queue as well as directly from the main
  * queue thread if the action queue is set to "direct".
  */
 static rsRetVal
@@ -1521,6 +1571,11 @@ processMsgMain(action_t *__restrict__ const pAction,
 
 	CHKiRet(prepareDoActionParams(pAction, pWti, pMsg, ttNow));
 
+	if(checkExternalStateFile(pAction) == RS_RET_SUSPENDED) {
+		DBGPRINTF("processMsgMain suspends via external state file\n");
+		actionSuspend(pAction, pWti);
+	}
+
 	if(pAction->isTransactional) {
 		pWti->actWrkrInfo[pAction->iActionNbr].pAction = pAction;
 		DBGPRINTF("action '%s': is transactional - executing in commit phase\n", pAction->pszName);
@@ -1747,7 +1802,7 @@ actionWriteToAction(action_t * const pAction, smsg_t *pMsg, wti_t * const pWti)
 		   }
 		if(pAction->iNbrNoExec < pAction->iExecEveryNthOccur - 1) {
 			++pAction->iNbrNoExec;
-			DBGPRINTF("action %p passed %d times to execution - less than neded - discarding\n",
+			DBGPRINTF("action %p passed %d times to execution - less than configured - discarding\n",
 			  pAction, pAction->iNbrNoExec);
 			FINALIZE;
 		} else {
@@ -1790,9 +1845,9 @@ finalize_it:
 /* Call configured action, most complex case with all features supported (and thus slow).
  * rgerhards, 2010-06-08
  */
-#ifndef _AIX
-#pragma GCC diagnostic ignored "-Wempty-body"
-#endif
+
+PRAGMA_DIAGNOSTIC_PUSH;
+PRAGMA_IGNORE_Wempty_body;
 static rsRetVal
 doSubmitToActionQComplex(action_t * const pAction, wti_t * const pWti, smsg_t *pMsg)
 {
@@ -1821,9 +1876,7 @@ finalize_it:
 
 	RETiRet;
 }
-#ifndef _AIX
-#pragma GCC diagnostic warning "-Wempty-body"
-#endif
+PRAGMA_DIAGNOSTIC_POP
 
 
 /* helper to activateActions, it activates a specific action.
@@ -1924,6 +1977,8 @@ actionApplyCnfParam(action_t * const pAction, struct cnfparamvals * const pvals)
 			continue; /* this is handled seperately during module select! */
 		} else if(!strcmp(pblk.descr[i].name, "action.errorfile")) {
 			pAction->pszErrFile = es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(pblk.descr[i].name, "action.externalstate.file")) {
+			pAction->pszExternalStateFile = es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(pblk.descr[i].name, "action.writeallmarkmessages")) {
 			pAction->bWriteAllMarkMsgs = pvals[i].val.d.n;
 		} else if(!strcmp(pblk.descr[i].name, "action.execonlyeverynthtime")) {
