@@ -154,6 +154,7 @@ struct instanceConf_s {
 	sbool fileNotFoundError;
 	int maxLinesAtOnce;
 	uint32_t trimLineOverBytes;
+	int msgFlag;
 	ruleset_t *pBindRuleset;	/* ruleset to bind listener to (use system default if unspecified) */
 	struct instanceConf_s *next;
 };
@@ -179,8 +180,10 @@ struct act_obj_s {
 #endif
 	time_t timeoutBase; /* what time to calculate the timeout against? */
 	/* file dynamic data */
+	char file_id[FILE_ID_HASH_SIZE]; /* file id for this entry, once we could obtain it */
 	int in_move;	/* workaround for inotify move: if set, state file must not be deleted */
 	ino_t ino;	/* current inode nbr */
+	int fd;		/* fd to file in order to obtain file_id (needs to be preserved across move) */
 	strm_t *pStrm;	/* its stream (NULL if not assigned) */
 	int nRecords; /**< How many records did we process before persisting the stream? */
 	ratelimit_t *ratelimiter;
@@ -316,7 +319,8 @@ static struct cnfparamdescr inppdescr[] = {
 	{ "statefile", eCmdHdlrString, CNFPARAM_DEPRECATED },
 	{ "readtimeout", eCmdHdlrPositiveInt, 0 },
 	{ "freshstarttail", eCmdHdlrBinary, 0},
-	{ "filenotfounderror", eCmdHdlrBinary, 0}
+	{ "filenotfounderror", eCmdHdlrBinary, 0},
+	{ "needparse", eCmdHdlrBinary, 0}
 };
 static struct cnfparamblk inppblk =
 	{ CNFPARAMBLK_VERSION,
@@ -680,6 +684,12 @@ act_obj_add(fs_edge_t *const edge, const char *const name, const int is_file,
 			}
 		}
 	}
+	DBGPRINTF("need to add new active object '%s' in '%s' - checking if accessible\n", name, edge->path);
+	const int fd = open(name, O_RDONLY | O_CLOEXEC);
+	if(fd < 0) {
+		LogMsg(errno, RS_RET_ERR, LOG_WARNING, "imfile: error accessing file '%s'", name);
+		FINALIZE;
+	}
 	DBGPRINTF("add new active object '%s' in '%s'\n", name, edge->path);
 	CHKmalloc(act = calloc(sizeof(act_obj_t), 1));
 	CHKmalloc(act->name = strdup(name));
@@ -690,6 +700,8 @@ act_obj_add(fs_edge_t *const edge, const char *const name, const int is_file,
 	}
 	act->edge = edge;
 	act->ino = ino;
+	act->fd = fd;
+	act->file_id[0] = '\0';
 	act->is_symlink = is_symlink;
 	if (source) { /* we are target of symlink */
 		CHKmalloc(act->source_name = strdup(source));
@@ -896,7 +908,7 @@ poll_timeouts(fs_edge_t *const edge)
 	if(edge->is_file) {
 		act_obj_t *act;
 		for(act = edge->active ; act != NULL ; act = act->next) {
-			if(strmReadMultiLine_isTimedOut(act->pStrm)) {
+			if(act->pStrm && strmReadMultiLine_isTimedOut(act->pStrm)) {
 				DBGPRINTF("timeout occured on %s\n", act->name);
 				pollFile(act);
 			}
@@ -955,6 +967,9 @@ act_obj_destroy(act_obj_t *const act, const int is_deleted)
 		wdmapDel(act->wd);
 	}
 	#endif
+	if(act->fd >= 0) {
+		close(act->fd);
+	}
 	#if defined(OS_SOLARIS) && defined (HAVE_PORT_SOURCE_FILE)
 	if(act->pfinf != NULL) {
 		free(act->pfinf->fobj.fo_name);
@@ -1208,24 +1223,24 @@ get_file_id_hash(const char *data, size_t lendata,
 
 /* this returns the file-id for a given file
  */
-static void ATTR_NONNULL(1, 2)
-getFileID(const act_obj_t *const act, char *const buf, const size_t lenbuf)
+static void ATTR_NONNULL(1)
+getFileID(act_obj_t *const act)
 {
-	*buf = '\0'; /* default: empty hash, only set if file has sufficient data */
-	const int fd = open(act->name, O_RDONLY | O_CLOEXEC);
-	if(fd >= 0) {
-		char filedata[FILE_ID_SIZE];
-		const int r = read(fd, filedata, FILE_ID_SIZE);
-		if(r == FILE_ID_SIZE) {
-			get_file_id_hash(filedata, sizeof(filedata), buf, lenbuf);
-		} else {
-			DBGPRINTF("getFileID partial or error read, ret %d\n", r);
-		}
-		close(fd);
-	} else {
-		DBGPRINTF("getFileID open %s failed\n", act->name);
+	if(act->file_id[0] != '\0') {
+		return; /* everything already done */
 	}
-	DBGPRINTF("getFileID for '%s', file_id_hash '%s'\n", act->name, buf);
+	assert(act->fd >= 0); /* fd must have been opened at act_obj_t creation! */
+	char filedata[FILE_ID_SIZE];
+	const int r = read(act->fd, filedata, FILE_ID_SIZE);
+	if(r == FILE_ID_SIZE) {
+		get_file_id_hash(filedata, sizeof(filedata), act->file_id, sizeof(act->file_id));
+		dbgprintf("file_id '%s' obtained, closing monitoring file handle\n", act->file_id);
+		close(act->fd); /* we will never go here! */
+		act->fd = -1;
+	} else {
+		DBGPRINTF("getFileID partial or error read, ret %d\n", r);
+	}
+	DBGPRINTF("getFileID for '%s', file_id_hash '%s'\n", act->name, act->file_id);
 }
 
 /* this generates a state file name suitable for the given file. To avoid
@@ -1242,7 +1257,7 @@ getStateFileName(const act_obj_t *const act,
 {
 	DBGPRINTF("getStateFileName for '%s'\n", act->name);
 	snprintf((char*)buf, lenbuf - 1, "imfile-state:%lld", (long long) act->ino);
-	DBGPRINTF("getStateFileName:  stat file name now is %s\n", buf);
+	DBGPRINTF("getStateFileName:  state file name now is %s\n", buf);
 	return buf;
 }
 
@@ -1304,6 +1319,8 @@ enqLine(act_obj_t *const act,
 		srSleep(inst->delay_perMsg % 1000000, inst->delay_perMsg / 1000000);
 	}
 
+	pMsg->msgFlags = pMsg->msgFlags | inst->msgFlag;
+
 	ratelimitAddMsg(act->ratelimiter, &act->multiSub, pMsg);
 finalize_it:
 	RETiRet;
@@ -1317,21 +1334,20 @@ openFileWithStateFile(act_obj_t *const act)
 	DEFiRet;
 	uchar pszSFNam[MAXFNAME];
 	uchar statefile[MAXFNAME];
-	char file_id[FILE_ID_HASH_SIZE];
 	int fd = -1;
 	const instanceConf_t *const inst = act->edge->instarr[0];// TODO: same file, multiple instances?
 
 	uchar *const statefn = getStateFileName(act, statefile, sizeof(statefile));
-	getFileID(act, file_id, sizeof(file_id));
+	getFileID(act);
 
-	getFullStateFileName(statefn, file_id, pszSFNam, sizeof(pszSFNam));
+	getFullStateFileName(statefn, act->file_id, pszSFNam, sizeof(pszSFNam));
 	DBGPRINTF("trying to open state for '%s', state file '%s'\n", act->name, pszSFNam);
 
 	/* check if the file exists */
 	fd = open((char*)pszSFNam, O_CLOEXEC | O_NOCTTY | O_RDONLY, 0600);
 	if(fd < 0) {
 		if(errno == ENOENT) {
-			if(file_id[0] != '\0') {
+			if(act->file_id[0] != '\0') {
 				const char *pszSFNamHash = strdup((const char*)pszSFNam);
 				CHKmalloc(pszSFNamHash);
 				DBGPRINTF("state file %s for %s does not exist - trying to see if "
@@ -1339,6 +1355,8 @@ openFileWithStateFile(act_obj_t *const act)
 				getFullStateFileName(statefn, "", pszSFNam, sizeof(pszSFNam));
 				fd = open((char*)pszSFNam, O_CLOEXEC | O_NOCTTY | O_RDONLY, 0600);
 				if(fd >= 0) {
+					dbgprintf("found inode-only state file, renaming it now that we "
+						"know the file_id, new name: %s\n", pszSFNamHash);
 					/* we now can use identify the file, so let's rename it */
 					if(rename((const char*)pszSFNam, pszSFNamHash) != 0) {
 						LogError(errno, RS_RET_IO_ERROR,
@@ -1505,6 +1523,7 @@ pollFileReal(act_obj_t *act, cstr_t **pCStr)
 {
 	int64 strtOffs;
 	DEFiRet;
+	int64_t startOffs = 0;
 	int nProcessed = 0;
 	regex_t *start_preg = NULL, *end_preg = NULL;
 
@@ -1521,6 +1540,7 @@ pollFileReal(act_obj_t *act, cstr_t **pCStr)
 	start_preg = (inst->startRegex == NULL) ? NULL : &inst->start_preg;
 	end_preg = (inst->endRegex == NULL) ? NULL : &inst->end_preg;
 
+	startOffs = act->pStrm->iCurrOffs;
 	/* loop below will be exited when strmReadLine() returns EOF */
 	while(glbl.GetGlobalInputTermState() == 0) {
 		if(inst->maxLinesAtOnce != 0 && nProcessed >= inst->maxLinesAtOnce)
@@ -1533,6 +1553,12 @@ pollFileReal(act_obj_t *act, cstr_t **pCStr)
 				inst->escapeLF, inst->discardTruncatedMsg, inst->msgDiscardingError, &strtOffs));
 		}
 		++nProcessed;
+		if(startOffs < FILE_ID_SIZE && act->pStrm->iCurrOffs >= FILE_ID_SIZE) {
+			dbgprintf("initiating state file write as sufficient data is now present; file=%s\n",
+				act->name);
+			persistStrmState(act);
+			startOffs = act->pStrm->iCurrOffs; /* disable check */
+		}
 		runModConf->bHadFileData = 1; /* this is just a flag, so set it and forget it */
 		CHKiRet(enqLine(act, *pCStr, strtOffs)); /* process line */
 		rsCStrDestruct(pCStr); /* discard string (must be done by us!) */
@@ -1607,6 +1633,7 @@ createInstance(instanceConf_t **const pinst)
 	inst->fileNotFoundError = 1;
 	inst->readTimeout = loadModConf->readTimeout;
 	inst->delay_perMsg = 0;
+	inst->msgFlag = 0;
 
 	/* node created, let's add to config */
 	if(loadModConf->tail == NULL) {
@@ -1751,6 +1778,7 @@ addInstance(void __attribute__((unused)) *pVal, uchar *pNewVal)
 	inst->addCeeTag = 0;
 	inst->bRMStateOnDel = 0;
 	inst->readTimeout = loadModConf->readTimeout;
+	inst->msgFlag = 0;
 
 	CHKiRet(checkInstance(inst));
 
@@ -1843,6 +1871,8 @@ CODESTARTnewInpInst
 			inst->nMultiSub = pvals[i].val.d.n;
 		} else if(!strcmp(inppblk.descr[i].name, "readtimeout")) {
 			inst->readTimeout = pvals[i].val.d.n;
+		} else if(!strcmp(inppblk.descr[i].name, "needparse")) {
+			inst->msgFlag = pvals[i].val.d.n ? NEEDS_PARSING : 0;
 		} else {
 			DBGPRINTF("program error, non-handled "
 			  "param '%s'\n", inppblk.descr[i].name);
@@ -2015,6 +2045,14 @@ ENDendCnfLoad
 BEGINcheckCnf
 	instanceConf_t *inst;
 CODESTARTcheckCnf
+	if(ustrlen(glbl.GetWorkDir()) == 0) {
+		/* this intentionally is an error message */
+		LogError(0, RS_RET_NO_WRKDIR_SET,
+			"imfile: no working directory set, imfile will create "
+			"state files in the current working directory (probably "
+			"the root dir). Use global(workDirectory=\"/some/path\") "
+			"to set the working directory");
+	}
 	for(inst = pModConf->root ; inst != NULL ; inst = inst->next) {
 		std_checkRuleset(pModConf, inst);
 	}
@@ -2261,10 +2299,8 @@ done:	return;
 
 
 /* Monitor files in inotify mode */
-#if !defined(_AIX)
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-align" /* TODO: how can we fix these warnings? */
-#endif
+PRAGMA_DIAGNOSTIC_PUSH
+PRAGMA_IGNORE_Wcast_align
 /* Problem with the warnings: they seem to stem back from the way the API is structured */
 static rsRetVal
 do_inotify(void)
@@ -2336,7 +2372,7 @@ finalize_it:
 	close(ino_fd);
 	RETiRet;
 }
-#pragma GCC diagnostic pop
+PRAGMA_DIAGNOSTIC_POP
 
 #else /* #if HAVE_INOTIFY_INIT */
 static rsRetVal
@@ -2543,13 +2579,12 @@ static rsRetVal ATTR_NONNULL()
 persistStrmState(act_obj_t *const act)
 {
 	DEFiRet;
-	char file_id[FILE_ID_HASH_SIZE];
 	uchar statefile[MAXFNAME];
 	uchar statefname[MAXFNAME];
 
 	uchar *const statefn = getStateFileName(act, statefile, sizeof(statefile));
-	getFileID(act, file_id, sizeof(file_id));
-	getFullStateFileName(statefn, file_id, statefname, sizeof(statefname));
+	getFileID(act);
+	getFullStateFileName(statefn, act->file_id, statefname, sizeof(statefname));
 	DBGPRINTF("persisting state for '%s', state file '%s'\n", act->name, statefname);
 
 	struct json_object *jval = NULL;
