@@ -25,58 +25,100 @@
  */
 #ifndef INCLUDED_RSYSLOG_H
 #define INCLUDED_RSYSLOG_H
-#ifndef _AIX
-#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
-#pragma GCC diagnostic ignored "-Wredundant-decls" // TODO: remove!
-#pragma GCC diagnostic ignored "-Wstrict-prototypes" // TODO: remove!
-#pragma GCC diagnostic ignored "-Wswitch-default" // TODO: remove!
-#if __GNUC__ >= 8
-/* GCC, starting at least with version 8, is now really overdoing with it's
- * warning messages. We turn those off that either cause an enormous amount
- * of false positives or flag perfectly legal code as problematic.
- */
-/* That one causes warnings when we use variable buffers for error
- * messages which may be truncated in the very unlikely case of all
- * vars using max value. If going over the max size, the engine will
- * most likely truncate due to max message size anyhow. Also, sizing
- * the buffers for max-max message size is a wast of (stack) memory.
- */
-#pragma GCC diagnostic ignored "-Wformat-truncation"
-/* The next one flags variable initializations within out exception handling
- * (iRet system) as problematic, even though variables are not used in those
- * cases. This would be a good diagnostic if gcc would actually check that
- * a variable is used uninitialized. Unfortunately it does not do that. But
- * the static analyzers we use as part of CI do, so we are covered in any
- * case.
- * Unfortunately ignoring this diagnostic leads to two more info lines
- * being emitted where nobody knows what the mean and why they appear :-(
- */
-#pragma GCC diagnostic ignored "-Wjump-misses-init"
-#endif /* if __GNUC__ >= 8 */
-#endif /* ifndef AIX */
+#ifdef __GNUC__
+	#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
+	#pragma GCC diagnostic ignored "-Wredundant-decls" // TODO: remove!
+	#pragma GCC diagnostic ignored "-Wstrict-prototypes" // TODO: remove!
+	#pragma GCC diagnostic ignored "-Wswitch-default" // TODO: remove!
+	#if __GNUC__ >= 8
+		/* GCC, starting at least with version 8, is now really overdoing with it's
+		 * warning messages. We turn those off that either cause an enormous amount
+		 * of false positives or flag perfectly legal code as problematic.
+		 */
+		/* That one causes warnings when we use variable buffers for error
+		 * messages which may be truncated in the very unlikely case of all
+		 * vars using max value. If going over the max size, the engine will
+		 * most likely truncate due to max message size anyhow. Also, sizing
+		 * the buffers for max-max message size is a wast of (stack) memory.
+		 */
+		#pragma GCC diagnostic ignored "-Wformat-truncation"
+		/* The next one flags variable initializations within out exception handling
+		 * (iRet system) as problematic, even though variables are not used in those
+		 * cases. This would be a good diagnostic if gcc would actually check that
+		 * a variable is used uninitialized. Unfortunately it does not do that. But
+		 * the static analyzers we use as part of CI do, so we are covered in any
+		 * case.
+		 * Unfortunately ignoring this diagnostic leads to two more info lines
+		 * being emitted where nobody knows what the mean and why they appear :-(
+		 */
+		#pragma GCC diagnostic ignored "-Wjump-misses-init"
+	#endif /* if __GNUC__ >= 8 */
 
-#include <pthread.h>
-#include "typedefs.h"
+	/* define a couple of attributes to improve cross-platform builds */
+	#if __GNUC__ > 6
+		#define CASE_FALLTHROUGH __attribute__((fallthrough));
+	#else
+		#define CASE_FALLTHROUGH
+	#endif
+
+	#define ATTR_NORETURN __attribute__ ((noreturn))
+	#define ATTR_UNUSED __attribute__((unused))
+	#define ATTR_NONNULL(...) __attribute__((nonnull(__VA_ARGS__)))
+
+#else /* ifdef __GNUC__ */
+
+	#define CASE_FALLTHROUGH
+	#define ATTR_NORETURN
+	#define ATTR_UNUSED
+	#define ATTR_NONNULL(...)
+
+#endif /* ifdef __GNUC__ */
 
 #if defined(_AIX)
 #include <sys/select.h>
 /* AIXPORT : start*/
 #define SRC_FD          13
 #define SRCMSG          (sizeof(srcpacket))
-extern int src_exists;
 #endif
 /* src end */
 
-/* define a couple of attributes to improve cross-platform builds */
-#if __GNUC__ > 6
-	#define CASE_FALLTHROUGH __attribute__((fallthrough));
-#else
-	#define CASE_FALLTHROUGH
-#endif
+#include <pthread.h>
+#include "typedefs.h"
 
-#define ATTR_NORETURN __attribute__ ((noreturn))
-#define ATTR_UNUSED __attribute__((unused))
-#define ATTR_NONNULL(...) __attribute__((nonnull(__VA_ARGS__)))
+#if defined(__GNUC__)
+	#define PRAGMA_INGORE_Wswitch_enum	_Pragma("GCC diagnostic ignored \"-Wswitch-enum\"")
+	#define PRAGMA_IGNORE_Wcast_align	_Pragma("GCC diagnostic ignored \"-Wcast-align\"")
+	#define PRAGMA_IGNORE_Wempty_body	_Pragma("GCC diagnostic ignored \"-Wempty-body\"")
+	#define PRAGMA_IGNORE_Wsign_compare	_Pragma("GCC diagnostic ignored \"-Wsign-compare\"")
+	#define PRAGMA_IGNORE_Wpragmas		_Pragma("GCC diagnostic ignored \"-Wpragmas\"")
+	#define PRAGMA_IGNORE_Wmissing_noreturn _Pragma("GCC diagnostic ignored \"-Wmissing-noreturn\"")
+	#define PRAGMA_IGNORE_Wexpansion_to_defined \
+						_Pragma("GCC diagnostic ignored \"-Wexpansion-to-defined\"")
+	#define PRAGMA_IGNORE_Wunknown_warning_option \
+						_Pragma("GCC diagnostic ignored \"-Wunknown-warning-option\"")
+	#define PRAGMA_IGNORE_Wunknown_attribute \
+						_Pragma("GCC diagnostic ignored \"-Wunknown-attribute\"")
+	#define PRAGMA_IGNORE_Wformat_nonliteral \
+						_Pragma("GCC diagnostic ignored \"-Wformat-nonliteral\"")
+	#define PRAGMA_IGNORE_Wdeprecated_declarations \
+						_Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+	#define PRAGMA_DIAGNOSTIC_PUSH		_Pragma("GCC diagnostic push")
+	#define PRAGMA_DIAGNOSTIC_POP		_Pragma("GCC diagnostic pop")
+#else
+	#define PRAGMA_INGORE_Wswitch_enum
+	#define PRAGMA_IGNORE_Wcast_align
+	#define PRAGMA_IGNORE_Wsign_compare
+	#define PRAGMA_IGNORE_Wformat_nonliteral
+	#define PRAGMA_IGNORE_Wpragmas
+	#define PRAGMA_IGNORE_Wmissing_noreturn
+	#define PRAGMA_IGNORE_Wempty_body
+	#define PRAGMA_IGNORE_Wdeprecated_declarations
+	#define PRAGMA_IGNORE_Wexpansion_to_defined
+	#define PRAGMA_IGNORE_Wunknown_attribute
+	#define PRAGMA_IGNORE_Wunknown_warning_option
+	#define PRAGMA_DIAGNOSTIC_PUSH
+	#define PRAGMA_DIAGNOSTIC_POP
+#endif
 
 /* ############################################################# *
  * #                 Some constant values                      # *
@@ -535,6 +577,10 @@ enum rsRetVal_				/** return value. All methods return this if not specified oth
 	RS_RET_POLL_ERR = -2444, /**< error in poll() system call */
 	RS_RET_OVERSIZE_MSG = -2445, /**< message is too long (above configured max) */
 	RS_RET_TLS_KEY_ERR = -2446, /**< TLS KEY has problems */
+	RS_RET_RABBITMQ_CONN_ERR = -2447, /**< RabbitMQ Connection error */
+	RS_RET_RABBITMQ_LOGIN_ERR = -2448, /**< RabbitMQ Login error */
+	RS_RET_RABBITMQ_CHANNEL_ERR = -2449, /**< RabbitMQ Connection error */
+	RS_RET_NO_WRKDIR_SET = -2450, /**< working directory not set, but desired by functionality */
 
 	/* RainerScript error messages (range 1000.. 1999) */
 	RS_RET_SYSVAR_NOT_FOUND = 1001, /**< system variable could not be found (maybe misspelled) */
@@ -673,7 +719,7 @@ struct actWrkrIParams {
 #  define  __attribute__(x)  /*NOTHING*/
 #endif
 
-#ifndef O_CLOEXEC
+#if !defined(O_CLOEXEC) && !defined(_AIX)
 /* of course, this limits the functionality... */
 #  define O_CLOEXEC 0
 #endif
