@@ -1,7 +1,7 @@
 #!/bin/bash
+# a case that actually caused a segfault
 # add 2017-11-06 by PascalWithopf, released under ASL 2.0
-
-. $srcdir/diag.sh init
+. ${srcdir:=.}/diag.sh init
 generate_conf
 add_conf '
 module(load="../plugins/imtcp/.libs/imtcp")
@@ -12,11 +12,9 @@ set $!x = "a";
 template(name="outfmt" type="string" string="-%$!%-\n")
 
 if $msg contains "msgnum:" then {
-	action(type="mmexternal" interface.input="fulljson"'
-add_conf "
-		binary=\"${srcdir}/testsuites/mmexternal-SegFault-mm-python.py\")"
-add_conf '
-	action(type="omfile" template="outfmt" file=`echo $RSYSLOG_OUT_LOG`)
+	action(type="mmexternal" interface.input="fulljson"
+		binary="'${srcdir}'/testsuites/mmexternal-SegFault-mm-python.py")
+	action(type="omfile" template="outfmt" file="'$RSYSLOG_OUT_LOG'")
 }
 '
 startup_vg
@@ -25,11 +23,6 @@ shutdown_when_empty
 wait_shutdown_vg
 check_exit_vg
 
-echo '-{ "x": "a", "sometag": "somevalue" }-' | cmp - $RSYSLOG_OUT_LOG
-if [ ! $? -eq 0 ]; then
-  echo "invalid response generated, $RSYSLOG_OUT_LOG is:"
-  cat $RSYSLOG_OUT_LOG
-  error_exit  1
-fi;
-
+export EXPECTED='-{ "x": "a", "sometag": "somevalue" }-'
+cmp_exact
 exit_test
