@@ -33,7 +33,7 @@ wait_shutdown
 # the first message number and calculate based on it what must be
 # present in the output file.
 presort
-let firstnum=$((10#`$RS_HEADCMD -n1 $RSYSLOG_DYNNAME.presort`))
+let firstnum=$((10#$($RS_HEADCMD -n1 $RSYSLOG_DYNNAME.presort)))
 echo "info: first message expected to be number $firstnum, using that value."
 seq_check $firstnum $((messages-1))
 exit_test
